@@ -29,11 +29,18 @@ class Charm(object):
         self._build = build
         self.vcs = self.get_vcs()
 
+    def is_git_branch(self):
+        return self.branch.startswith('git') or \
+            self.branch.find("review.openstack.org") != -1 or \
+            "github.com" in self.branch or \
+            "git.launchpad.net" in self.branch or \
+            os.path.exists(os.path.join(self.branch, '.git'))
+
     def get_vcs(self):
         if not self.branch:
             return None
-        if self.branch.startswith('git') or 'github.com' in self.branch or \
-                os.path.exists(os.path.join(self.branch, '.git')):
+
+        if self.is_git_branch():
             return Git(self.path, self.branch, self.log)
         elif self.branch.startswith("bzr") or self.branch.startswith('lp:') \
                 or os.path.exists(os.path.join(self.branch, '.bzr')) \
@@ -69,6 +76,9 @@ class Charm(object):
             else:
                 series = deploy_series
             charm_path = path_join(repo_path, series, name)
+        elif os.path.isabs(name):
+            # charm points to an absolute local path
+            charm_path = name.rstrip(os.path.sep)
         elif 'series' in data:
             series = data['series']
             charm_path = path_join(repo_path, series, name)
@@ -82,15 +92,36 @@ class Charm(object):
             cls.log.error(
                 "Service: %s has both charm url: %s and branch: %s specified",
                 name, store_url, branch)
-        if not store_url and not branch:
-            cls.log.error(
-                "Service: %s has neither charm url or branch specified",
-                name)
-        elif not store_url:
+        if not store_url:
             build = data.get('build', '')
 
         return cls(name, charm_path, branch, rev, build, store_url)
 
+    def is_absolute(self):
+        """Charm config points to an absolute path on disk.
+
+        """
+        return os.path.isabs(self.name)
+
+    @property
+    def repo_path(self):
+        """The Juju repository path in which this charm resides.
+
+        For most charms this returns None, leaving the repo path to be
+        determined by the Deployment that deploys the charm.
+
+        For charms at an absolute path, however, the repo path is by
+        definition the directory two levels up from the charm. (And the
+        series directory is one level up.) This allows us to deploy
+        charms from anywhere on the filesystem without first gathering them
+        under one repository path.
+
+        """
+        if self.is_absolute():
+            d = os.path.dirname
+            return d(d(self.path))
+        return None
+
     def is_local(self):
         if self._charm_url:
             if self._charm_url.startswith('cs:'):
@@ -117,12 +148,14 @@ class Charm(object):
         self.log.debug("Building charm %s with %s", self.path, self._build)
         _check_call([self._build], self.log,
                     "Charm build failed %s @ %s", self._build, self.path,
-                    cwd=self.path)
+                    cwd=self.path, shell=True)
 
     def fetch(self):
         if self._charm_url:
             self._fetch_store_charm()
             return
+        elif self.is_absolute():
+            return
         elif not self.branch:
             self.log.warning("Invalid charm specification %s", self.name)
             return
@@ -140,7 +173,7 @@ class Charm(object):
 
     @property
     def series_path(self):
-        if not self.is_local():
+        if self.is_absolute() or not self.is_local():
             return None
         return os.path.dirname(self.path)
 
