@@ -50,10 +50,10 @@ def setup_parser():
         default=False)
     parser.add_argument(
         '-T', '--terminate-machines',
-        help=('Terminate all machines but the bootstrap node.  '
-              'Destroy any services that exist on each'),
-        dest="terminate_machines", action="store_true",
-        default=False)
+        help=('Terminate all machines but the bootstrap node. '
+              'Destroy any services that exist on each. '
+              'Use -TT to forcefully terminate.'),
+        dest="terminate_machines", action="count", default=0)
     parser.add_argument(
         '-t', '--timeout',
         help='Timeout (sec) for entire deployment (45min default)',
@@ -118,6 +118,11 @@ def setup_parser():
         help=('Number of seconds to wait before checking for '
               'relation errors after all relations have been added '
               'and subordinates started. (default: 60)'))
+    parser.add_argument(
+        '-n', '--no-relations',
+        default=False, dest='no_relations', action='store_true',
+        help=('Do not add relations to environment, just services/units '
+              '(default: False)'))
     parser.add_argument("--description", help=argparse.SUPPRESS,
                         action="store_true")
     parser.add_argument("deployment", nargs="?")
@@ -166,9 +171,10 @@ def run():
     if options.destroy_services or options.terminate_machines:
         log.info("Resetting environment...")
         env.connect()
-        env.reset(terminate_machines=options.terminate_machines,
+        env.reset(terminate_machines=bool(options.terminate_machines),
                   terminate_delay=options.deploy_delay,
-                  watch=options.watch)
+                  watch=options.watch,
+                  force_terminate=options.terminate_machines > 1)
         log.info("Environment reset in %0.2f", time.time() - start_time)
         sys.exit(0)
 
