@@ -19,6 +19,7 @@ class ConfigStack(object):
     def __init__(self, config_files, cli_series=None):
         self.config_files = config_files
         self.cli_series = cli_series
+        self.version = 3
         self.data = {}
         self.yaml = {}
         self.include_dirs = []
@@ -42,30 +43,40 @@ class ConfigStack(object):
 
         with open(config_file) as fh:
             try:
-                self.yaml[config_file] = yaml_load(fh.read())
+                yaml_result = yaml_load(fh.read())
             except Exception, e:
                 self.log.warning(
                     "Couldn't load config file @ %r, error: %s:%s",
                     config_file, type(e), e)
                 raise
 
+        # Check if this is a v4 bundle.
+        services = yaml_result.get('services')
+        if isinstance(services, dict) and 'services' not in services:
+            self.version = 4
+            yaml_result = {config_file: yaml_result}
+
+        self.yaml[config_file] = yaml_result
+
         return self.yaml[config_file]
 
     def keys(self):
         return sorted(self.data)
 
     def get(self, key):
-        if not key in self.data:
+        if key not in self.data:
             self.log.warning("Deployment %r not found. Available %s",
                              key, ", ".join(self.keys()))
             raise ErrorExit()
         deploy_data = self.data[key]
-        deploy_data = self._resolve_inherited(deploy_data)
+        if self.version < 4:
+            deploy_data = self._resolve_inherited(deploy_data)
         if self.cli_series:
             deploy_data['series'] = self.cli_series
         return Deployment(
             key, deploy_data, self.include_dirs,
-            repo_path=os.environ.get("JUJU_REPOSITORY", ""))
+            repo_path=os.environ.get("JUJU_REPOSITORY", ""),
+            version=self.version)
 
     def load(self):
         data = {}
@@ -88,7 +99,7 @@ class ConfigStack(object):
         return parents
 
     def _resolve_inherited(self, deploy_data):
-        if not 'inherits' in deploy_data:
+        if 'inherits' not in deploy_data:
             return deploy_data
         inherits = parents = self._inherits(deploy_data)
         for parent_name in parents:
