@@ -7,20 +7,22 @@ import yaml
 
 from .charm import Charm
 from .feedback import Feedback
-from .service import Service, ServiceUnitPlacement
+from .service import Service, ServiceUnitPlacementV3, ServiceUnitPlacementV4
 from .relation import Endpoint
-from .utils import path_join, yaml_dump, ErrorExit, resolve_include
+from .utils import path_join, yaml_dump, ErrorExit, resolve_include, x_in_y
 
 
 class Deployment(object):
 
     log = logging.getLogger("deployer.deploy")
 
-    def __init__(self, name, data, include_dirs, repo_path=""):
+    def __init__(self, name, data, include_dirs, repo_path="", version=3):
         self.name = name
         self.data = data
         self.include_dirs = include_dirs
         self.repo_path = repo_path
+        self.version = version
+        self.machines = {}
 
     @property
     def series(self):
@@ -36,7 +38,7 @@ class Deployment(object):
         pprint.pprint(self.data)
 
     def get_service(self, name):
-        if not name in self.data['services']:
+        if name not in self.data['services']:
             return
         return Service(name, self.data['services'][name])
 
@@ -44,17 +46,59 @@ class Deployment(object):
         services = []
         for name, svc_data in self.data.get('services', {}).items():
             services.append(Service(name, svc_data))
-        services.sort(self._placement_sort)
+        if self.version == 3:
+            # Sort unplaced units first, then sort by name for placed units.
+            services.sort(key=lambda svc: (bool(svc.unit_placement), svc.name))
+        else:
+            services.sort(self._machines_placement_sort)
         return services
 
+    def set_machines(self, machines):
+        """Set a dict of machines, mapping from the names in the machine spec
+        to the machine names in the environment status.
+        """
+        self.machines = machines
+
+    def get_machine(self, id):
+        """Return a dict containing machine options
+        None is returned if the machine doesn't exists in
+        the bundle YAML."""
+        return self.get_machines().get(id, None)
+
+    def get_machines(self):
+        """Return a dict mapping machine names to machine options.
+
+        An empty dict is returned if no machines are defined in the
+        bundle YAML.
+        """
+        machines = {}
+        for key, machine in self.data.get('machines', {}).items():
+            machines[str(key)] = machine
+        return machines
+
     def get_service_names(self):
         """Return a sequence of service names for this deployment."""
         return self.data.get('services', {}).keys()
 
     @staticmethod
-    def _placement_sort(svc_a, svc_b):
+    def _machines_placement_sort(svc_a, svc_b):
+        """Sort machines with machine placement in mind.
+
+        If svc_a is colocated alongside svc_b, svc_b needs to be deployed
+        first, so that it exists by the time svc_a is deployed, and vice
+        versa; this sorts first based on this fact, then secondly based on
+        whether or not the service has a unit placement, and then finally
+        based on the name of the service.
+        """
         if svc_a.unit_placement:
             if svc_b.unit_placement:
+                # Check for colocation.  This naively assumes that there is no
+                # circularity in placements.
+                if x_in_y(svc_b, svc_a):
+                    return 1
+                if x_in_y(svc_a, svc_b):
+                    return -1
+                # If no colocation exists, simply compare names.
                 return cmp(svc_a.name, svc_b.name)
             return 1
         if svc_b.unit_placement:
@@ -64,7 +108,11 @@ class Deployment(object):
     def get_unit_placement(self, svc, status):
         if isinstance(svc, (str, unicode)):
             svc = self.get_service(svc)
-        return ServiceUnitPlacement(svc, self, status)
+        if self.version == 3:
+            return ServiceUnitPlacementV3(svc, self, status)
+        else:
+            return ServiceUnitPlacementV4(svc, self, status,
+                                          machines_map=self.machines)
 
     def get_relations(self):
         if 'relations' not in self.data:
@@ -161,7 +209,7 @@ class Deployment(object):
             for svc_name, svc_data in self.data['services'].items():
                 charm = self.get_charm_for(svc_name)
                 if k in charm.config:
-                    if 'options' not in svc_data:
+                    if not svc_data.get('options'):
                         svc_data['options'] = {}
                     svc_data['options'][k] = v
                     found = True
