@@ -1,7 +1,10 @@
 import time
 
 from .base import BaseEnvironment
-from ..utils import ErrorExit
+from ..utils import (
+    ErrorExit,
+    parse_constraints,
+)
 
 from jujuclient import (
     EnvError,
@@ -24,6 +27,20 @@ class GoEnvironment(BaseEnvironment):
         self.api_endpoint = endpoint
         self.client = None
 
+    def add_machine(self, series="", constraints={}):
+        """Add a top level machine to the Juju environment.
+
+        Use the given series and constraints.
+        Return the machine identifier (e.g. "1").
+
+        series: a string such as 'precise' or 'trusty'.
+        constraints: a map of constraints (such as mem, arch, etc.) which
+          can be parsed by utils.parse_constraints
+        """
+        return self.client.add_machine(
+            series=series,
+            constraints=parse_constraints(constraints))['Machine']
+
     def add_unit(self, service_name, machine_spec):
         return self.client.add_unit(service_name, machine_spec)
 
@@ -65,7 +82,9 @@ class GoEnvironment(BaseEnvironment):
     def reset(self,
               terminate_machines=False,
               terminate_delay=0,
-              timeout=360, watch=False):
+              timeout=360,
+              watch=False,
+              force_terminate=False):
         """Destroy/reset the environment."""
         status = self.status()
         destroyed = False
@@ -78,8 +97,10 @@ class GoEnvironment(BaseEnvironment):
             # Mark any errors as resolved so destruction can proceed.
             self.resolve_errors()
 
-            # Wait for units
-            self.wait_for_units(timeout, goal_state='removed', watch=watch)
+            if not (terminate_machines and force_terminate):
+                # Wait for units to be removed. Only necessary if we're not
+                # force-terminating machines.
+                self.wait_for_units(timeout, goal_state='removed', watch=watch)
 
         # The only value to not terminating is keeping the data on the
         # machines around.
@@ -87,13 +108,15 @@ class GoEnvironment(BaseEnvironment):
             self.log.info(
                 " warning: juju-core machines are not reusable for units")
             return
-        self._terminate_machines(status, watch, terminate_delay)
+        self._terminate_machines(
+            status, watch, terminate_delay, force_terminate)
 
-    def _terminate_machines(self, status, watch, terminate_wait):
+    def _terminate_machines(self, status, watch, terminate_wait, force):
         """Terminate all machines, optionally wait for termination.
         """
         # Terminate machines
-        self.log.debug(" Terminating machines")
+        self.log.debug(
+            " Terminating machines %s", 'forcefully' if force else '')
 
         # Don't bother if there are no service unit machines
         if len(status['machines']) == 1:
@@ -120,7 +143,7 @@ class GoEnvironment(BaseEnvironment):
         machines.sort(machine_sort)
 
         for mid in machines:
-            self._terminate_machine(mid, container_hosts)
+            self._terminate_machine(mid, container_hosts, force=force)
 
         if containers:
             watch = self.client.get_watch(120)
@@ -128,20 +151,20 @@ class GoEnvironment(BaseEnvironment):
                 watch, containers).run(self._delta_event_log)
 
         for mid in container_hosts:
-            self._terminate_machine(mid)
+            self._terminate_machine(mid, force=force)
 
         if terminate_wait:
             self.log.info("  Waiting for machine termination")
             callback = watch and self._delta_event_log or None
-            self.client.wait_for_no_machines(None, callback)
+            self.client.wait_for_no_machines(terminate_wait, callback)
 
-    def _terminate_machine(self, mid, container_hosts=()):
+    def _terminate_machine(self, mid, container_hosts=(), force=False):
         if mid == "0":
             return
         if mid in container_hosts:
             return
         self.log.debug("  Terminating machine %s", mid)
-        self.terminate_machine(mid)
+        self.terminate_machine(mid, force=force)
 
     def _check_timeout(self, etime):
         w_timeout = etime - time.time()
@@ -196,8 +219,15 @@ class GoEnvironment(BaseEnvironment):
             else:
                 return
 
-    def set_annotation(self, svc_name, annotation):
-        return self.client.set_annotation(svc_name, 'service', annotation)
+    def set_annotation(self, entity_name, annotation, entity_type='service'):
+        """Set an annotation on an entity.
+
+        entity_name: the name of the entity (machine, service, etc.) to
+          annotate.
+        annotation: a dict of key/value pairs to set on the entity.
+        entity_type: the type of entity (machine, service, etc.) to annotate.
+        """
+        return self.client.set_annotation(entity_name, entity_type, annotation)
 
     def status(self):
         return self.client.get_stat()
