@@ -37,8 +37,8 @@ def get_default_guiserver_options():
 class GUIDeployment(Deployment):
     """Handle bundle deployments requested by the GUI server."""
 
-    def __init__(self, name, data):
-        super(GUIDeployment, self).__init__(name, data, [])
+    def __init__(self, name, data, version):
+        super(GUIDeployment, self).__init__(name, data, [], version=version)
 
     def _handle_feedback(self, feedback):
         """Raise a DeploymentError if the given feedback includes errors.
@@ -72,9 +72,9 @@ def _validate(env, bundle):
         raise ValueError(error)
 
 
-def validate(apiurl, password, bundle):
+def validate(apiurl, username, password, bundle):
     """Validate a bundle."""
-    env = GUIEnvironment(apiurl, password)
+    env = GUIEnvironment(apiurl, username, password)
     env.connect()
     try:
         _validate(env, bundle)
@@ -82,10 +82,17 @@ def validate(apiurl, password, bundle):
         env.close()
 
 
-def import_bundle(apiurl, password, name, bundle, options):
-    """Import a bundle."""
-    env = GUIEnvironment(apiurl, password)
-    deployment = GUIDeployment(name, bundle)
+def import_bundle(apiurl, username, password, name, bundle, version, options):
+    """Import a bundle.
+
+    To connect to the Juju environment, use the given API URL, user name and
+    password. The name and bundle arguments are used to deploy the bundle. The
+    version argument specifies whether the given bundle content uses the
+    legacy v3 or the new v4 bundle syntax. The given options are used to
+    start the bundle deployment process.
+    """
+    env = GUIEnvironment(apiurl, username, password)
+    deployment = GUIDeployment(name, bundle, version=version)
     importer = Importer(env, deployment, options)
     env.connect()
     # The Importer tries to retrieve the Juju home from the JUJU_HOME
