@@ -9,6 +9,30 @@ from deployer.vcs import Git as BaseGit
 from .base import Base
 
 
+class StoreCharmTest(Base):
+    def setUp(self):
+        self.charm = Charm(
+            'cs:trusty/ubuntu', None, None, None, None)
+
+    def test_is_absolute(self):
+        self.assertFalse(self.charm.is_absolute())
+
+    def test_repo_path(self):
+        self.assertIsNone(self.charm.repo_path)
+
+
+class AbsolutePathCharmTest(Base):
+    def setUp(self):
+        self.charm = Charm(
+            '/tmp/trusty/ubuntu', '/tmp/trusty/ubuntu', None, None, None)
+
+    def test_is_absolute(self):
+        self.assertTrue(self.charm.is_absolute())
+
+    def test_repo_path(self):
+        self.assertEqual('/tmp', self.charm.repo_path)
+
+
 class Bzr(BaseBzr):
 
     def __init__(self, path):
@@ -267,6 +291,23 @@ class GitCharmTest(Base):
         charm.fetch()
         self.assertEqual(charm.vcs.get_cur_rev(), self.tagged_revision)
 
+    def test_vcs_fetch_with_refspec(self):
+        self.setup_vcs_charm()
+        params = dict(self.charm_data)
+        params['branch'] = \
+            "https://review.openstack.org/openstack/charm-ceilometer" + \
+            "#changeref=286669/1"
+
+        charm = Charm.from_service(
+            "scratch", self.repo_path, "trusty", params)
+        charm.fetch()
+
+        self.assertEqual(charm.vcs.get_remote_origin(),
+                         "https://review.openstack.org/" +
+                         "openstack/charm-ceilometer")
+        self.assertEqual(charm.vcs.extended_options,
+                         {'changeref': "286669/1"})
+
     def test_charm_vcs_unknown(self):
         branch = self.mkdir()
         params = {
