@@ -1,4 +1,5 @@
 import logging
+import mock
 import os
 import tempfile
 import yaml
@@ -27,6 +28,7 @@ class ConfigTest(Base):
         self.assertRaises(ErrorExit, config.get, 'zeeland')
         result = config.get("openstack-precise-ec2")
         self.assertTrue(isinstance(result, Deployment))
+        self.assertEqual(config.version, 3)
 
     def test_config(self):
         config = ConfigStack([
@@ -38,6 +40,21 @@ class ConfigTest(Base):
             [u'my-files-frontend-dev', u'wordpress'])
         deployment = config.get("wordpress")
         self.assertTrue(deployment)
+        self.assertEqual(config.version, 3)
+
+    def test_config_v4(self):
+        config = ConfigStack([
+            os.path.join(self.test_data_dir, 'v4', 'simple.yaml')])
+        config.load()
+        self.assertEqual(
+            config.keys(),
+            [os.path.join(self.test_data_dir, 'v4', 'simple.yaml')])
+        with mock.patch('deployer.config.ConfigStack._resolve_inherited') \
+                as mock_resolve:
+            deployment = config.get(config.keys()[0])
+            self.assertTrue(deployment)
+            self.assertFalse(mock_resolve.called)
+        self.assertEqual(config.version, 4)
 
     def test_config_include_file(self):
         config = ConfigStack([
