@@ -113,10 +113,20 @@ class ConstraintsTest(Base):
 
     def test_non_numerics_are_not_converted(self):
         # Constraints that expect strings are not affected by the parsing.
-        keys = ['arch', 'container', 'tags']
+        keys = ['arch', 'container']
         for k in keys:
             value = {
                 k: '1T',
             }
             result = parse_constraints(value)
             self.assertEqual(result[k], '1T')
+
+    def test_tags(self):
+        # Tags are provided as a list of strings.
+        constraints = parse_constraints({'tags': 'foo,bar'})
+        self.assertEqual({'tags': ['foo', 'bar']}, constraints)
+
+    def test_single_tag(self):
+        # A single tag is converted to a list.
+        constraints = parse_constraints({'tags': 'foo'})
+        self.assertEqual({'tags': ['foo']}, constraints)
