@@ -11,7 +11,10 @@ import yaml
 
 from deployer import guiserver
 from deployer.feedback import Feedback
-from deployer.tests.base import TEST_OFFLINE
+from deployer.tests.base import (
+    patch_env_status,
+    skip_if_offline,
+)
 
 
 class TestGetDefaultGuiserverOptions(unittest.TestCase):
@@ -26,9 +29,9 @@ class TestGetDefaultGuiserverOptions(unittest.TestCase):
             'bootstrap', 'branch_only', 'configs', 'debug', 'deploy_delay',
             'deployment', 'description', 'destroy_services', 'diff',
             'find_service', 'ignore_errors', 'juju_env', 'list_deploys',
-            'no_local_mods', 'overrides', 'rel_wait', 'retry_count', 'series',
-            'skip_unit_wait', 'terminate_machines', 'timeout', 'update_charms',
-            'verbose', 'watch'
+            'no_local_mods', 'no_relations', 'overrides', 'rel_wait',
+            'retry_count', 'series', 'skip_unit_wait', 'terminate_machines',
+            'timeout', 'update_charms', 'verbose', 'watch'
         ])
         self.assertEqual(expected_keys, set(self.options.__dict__.keys()))
 
@@ -80,7 +83,7 @@ class TestGUIDeployment(unittest.TestCase):
 
     def setUp(self):
         # Set up a GUIDeployment instance and a Feedback object.
-        self.deployment = guiserver.GUIDeployment('mybundle', 'mydata')
+        self.deployment = guiserver.GUIDeployment('mybundle', 'mydata', 4)
         self.feedback = Feedback()
 
     def test_valid_deployment(self):
@@ -107,6 +110,7 @@ class DeployerFunctionsTestMixin(object):
     """Base set up for the functions that make use of the juju-deployer."""
 
     apiurl = 'wss://api.example.com:17070'
+    username = 'who'
     password = 'Secret!'
     name = 'mybundle'
     bundle = yaml.safe_load("""
@@ -144,7 +148,8 @@ class DeployerFunctionsTestMixin(object):
         services, the environment is instantiated, connected, env.status is
         called and then the connection is closed.
         """
-        mock_environment.assert_called_once_with(self.apiurl, self.password)
+        mock_environment.assert_called_once_with(
+            self.apiurl, self.username, self.password)
         mock_env_instance = mock_environment()
         mock_env_instance.connect.assert_called_once_with()
         mock_env_instance.status.assert_called_once_with()
@@ -170,14 +175,14 @@ class DeployerFunctionsTestMixin(object):
         mock_env_instance.close.assert_called_once_with()
 
 
-@unittest.skipIf(TEST_OFFLINE,
-                 "Requires configured bzr launchpad id and network access")
+@skip_if_offline
 @mock.patch('deployer.guiserver.GUIEnvironment')
 class TestValidate(DeployerFunctionsTestMixin, unittest.TestCase):
 
     def test_validation(self, mock_environment):
         # The validation is correctly run.
-        guiserver.validate(self.apiurl, self.password, self.bundle)
+        guiserver.validate(
+            self.apiurl, self.username, self.password, self.bundle)
         # The environment is correctly instantiated and used.
         self.check_environment_life(mock_environment)
 
@@ -185,11 +190,11 @@ class TestValidate(DeployerFunctionsTestMixin, unittest.TestCase):
         # The validation fails if the bundle includes a service name already
         # present in the Juju environment.
         with self.assert_overlapping_services(mock_environment):
-            guiserver.validate(self.apiurl, self.password, self.bundle)
+            guiserver.validate(
+                self.apiurl, self.username, self.password, self.bundle)
 
 
-@unittest.skipIf(TEST_OFFLINE,
-                 "Requires configured bzr launchpad id and network access")
+@skip_if_offline
 @mock.patch('deployer.guiserver.GUIEnvironment')
 class TestImportBundle(DeployerFunctionsTestMixin, unittest.TestCase):
 
@@ -208,10 +213,11 @@ class TestImportBundle(DeployerFunctionsTestMixin, unittest.TestCase):
             finally:
                 del os.environ['JUJU_HOME']
 
-    def import_bundle(self):
+    def import_bundle(self, version=4):
         """Call the import_bundle function."""
         guiserver.import_bundle(
-            self.apiurl, self.password, self.name, self.bundle, self.options)
+            self.apiurl, self.username, self.password, self.name, self.bundle,
+            version, self.options)
 
     def cleanup_series_path(self):
         """Remove the series path created by the Deployment object."""
@@ -265,12 +271,15 @@ class TestImportBundle(DeployerFunctionsTestMixin, unittest.TestCase):
     def test_importer_behavior(self, mock_sleep, mock_environment):
         # The importer executes the expected environment calls.
         self.addCleanup(self.cleanup_series_path)
+        patch_env_status(mock_environment(), {'mysql': 1, 'wordpress': 1})
+        mock_environment.reset_mock()
         with self.patch_juju_home():
-            self.import_bundle()
+            self.import_bundle(version=3)
         mock_sleep.assert_has_calls([mock.call(5.1), mock.call(60)])
         # If any of the calls below fails, then we have to change the
         # signatures of deployer.guiserver.GUIEnvironment methods.
-        mock_environment.assert_called_once_with(self.apiurl, self.password)
+        mock_environment.assert_called_once_with(
+            self.apiurl, self.username, self.password)
         mock_environment().assert_has_calls([
             mock.call.connect(),
             mock.call.status(),
@@ -285,8 +294,7 @@ class TestImportBundle(DeployerFunctionsTestMixin, unittest.TestCase):
                 None, 1, None),
             mock.call.set_annotation(
                 'wordpress', {'gui-y': '414.547', 'gui-x': '425.347'}),
-            mock.call.add_units('mysql', 2),
-            mock.call.add_units('wordpress', 1),
+            mock.call.add_units('mysql', 1),
             mock.call.add_relation('mysql:db', 'wordpress:db'),
             mock.call.close(),
         ], any_order=True)
@@ -305,12 +313,13 @@ class TestImportBundle(DeployerFunctionsTestMixin, unittest.TestCase):
                 },
             },
         }
+        version = 4
         self.addCleanup(self.cleanup_series_path)
         with self.patch_juju_home():
             with self.assertRaises(guiserver.DeploymentError) as cm:
                 guiserver.import_bundle(
-                    self.apiurl, self.password, self.name, bundle,
-                    self.options)
+                    self.apiurl, self.username, self.password, self.name,
+                    bundle, version, self.options)
         expected_errors = set([
             'Invalid config charm cs:precise/wordpress-20 no-such-option=42',
             'Invalid config charm cs:precise/mysql-28 bad=wolf',
