@@ -68,6 +68,13 @@ class UtilTests(Base):
         # 1 failure + 3 retries
         self.assertEquals(len(check_output.call_args_list), 3)
 
+    def test_check_call_uses_shell(self):
+        cmd = 'echo "foo"'
+        self.assertRaises(
+            OSError, _check_call, params=[cmd], log=MagicMock())
+        output = _check_call(params=[cmd], log=MagicMock(), shell=True)
+        self.assertEqual(output, "foo\n")
+
 
 class TestMkdir(Base):
 
@@ -158,7 +165,7 @@ class TestCharmRevisioning(Base):
             with self.assertRaises(DeploymentError) as exc:
                 get_qualified_charm_url('cs:precise/mysql')
             expected = ('HTTP Error 404: '
-                        'Bad Earl (https://store.juju.ubuntu.com/charm-info'
+                        'Bad Earl (https://api.jujucharms.com/charmstore/charm-info'
                         '?charms=cs:precise/mysql)')
             self.assertEqual([expected], exc.exception.message)
 
@@ -170,6 +177,6 @@ class TestCharmRevisioning(Base):
             with self.assertRaises(DeploymentError) as exc:
                 get_qualified_charm_url('cs:precise/mysql')
             expected = ('<urlopen error Hinky URL> '
-                        '(https://store.juju.ubuntu.com/charm-info'
+                        '(https://api.jujucharms.com/charmstore/charm-info'
                         '?charms=cs:precise/mysql)')
             self.assertEqual([expected], exc.exception.message)
