@@ -53,7 +53,7 @@ class DeploymentError(Exception):
         return '\n'.join(self.errors)
 
 
-STORE_URL = "https://store.juju.ubuntu.com"
+STORE_URL = "https://api.jujucharms.com/charmstore"
 
 
 # Utility functions
@@ -208,6 +208,7 @@ def parse_constraints(value):
         return value
     constraints_with_units = ['mem', 'root-disk']
     numerics = ['cpu-cores', 'cpu-power'] + constraints_with_units
+    lists = ['tags']
     constraints = {}
     if isinstance(value, dict):
         constraints.update(value)
@@ -218,7 +219,12 @@ def parse_constraints(value):
             constraints[k] = v
 
     for k, v in constraints.items():
-        if k in numerics:
+        if k in lists:
+            v = v.split(',')
+        elif k in numerics:
+            # Currently numeric constraints are never passed as slices. For
+            # this reason, it is safe to assume that a numeric value is always
+            # a string and never a list.
             units = 1
             if k in constraints_with_units:
                 v, units = _convert_units_specifier(v)
@@ -242,6 +248,7 @@ def _get_juju_home():
 def _check_call(params, log, *args, **kw):
     max_retry = kw.get('max_retry', None)
     cur = kw.get('cur_try', 1)
+    shell = kw.get('shell', False)
     try:
         cwd = abspath(".")
         if 'cwd' in kw:
@@ -250,7 +257,7 @@ def _check_call(params, log, *args, **kw):
         if 'stderr' in kw:
             stderr = kw['stderr']
         output = subprocess.check_output(
-            params, cwd=cwd, stderr=stderr, env=os.environ)
+            params, cwd=cwd, stderr=stderr, env=os.environ, shell=shell)
     except subprocess.CalledProcessError, e:
         if 'ignoreerr' in kw:
             return
@@ -372,3 +379,23 @@ def get_env_name(param_env_name):
         if not 'default' in conf:
             raise ValueError("No Environment specified")
         return conf['default']
+
+
+def x_in_y(x, y):
+    """Check to see if the second argument is named in the first
+    argument's unit placement spec.
+
+    Both arguments provided are services with unit placement directives.
+    If the first service appears in the second service's unit placement,
+    either colocated on a default unit, colocated with a specific unit,
+    or containerized alongside that service, then True is returned, False
+    otherwise.
+    """
+    for placement in y.unit_placement:
+        if ':' in placement:
+            _, placement = placement.split(':')
+        if '/' in placement:
+            placement, _ = placement.split('/')
+        if x.name == placement:
+            return True
+    return False
