@@ -1,5 +1,6 @@
 import subprocess
 import os
+import re
 
 from bzrlib.workingtree import WorkingTree
 
@@ -20,7 +21,12 @@ class Vcs(object):
     def __init__(self, path, origin, log):
         self.path = path
         self.log = log
-        self.origin = origin
+        self.extended_options = self.get_extended_options(origin)
+
+        if self.extended_options:
+            self.origin = origin.split("#")[0]
+        else:
+            self.origin = origin
 
     def _call(self, args, error_msg, cwd=None, stderr=()):
         try:
@@ -29,7 +35,6 @@ class Vcs(object):
             output = subprocess.check_output(
                 args, cwd=cwd or self.path, stderr=stderr)
         except subprocess.CalledProcessError, e:
-            #print "vcs err", " ".join(args), "[dir: %s]" % cwd, e.output
             self.log.error(error_msg % self.get_err_msg_ctx(e))
             raise ErrorExit()
         return output.strip()
@@ -42,6 +47,20 @@ class Vcs(object):
             'output': e.output,
             'vcs': self.__class__.__name__.lower()}
 
+    def get_extended_options(self, origin):
+        regexp = re.compile(r"[\?#&](?P<name>[^&=]+)=(?P<value>[^&=]+)")
+        matched = regexp.findall(origin)
+
+        if matched:
+            ret = dict()
+            for option in matched:
+                (name, value) = option
+                if name in ret:
+                    raise Exception("%s option already defined" % name)
+                ret[name] = value
+            return ret
+        return {}
+
     def get_cur_rev(self):
         raise NotImplementedError()
 
@@ -99,8 +118,31 @@ class Git(Vcs):
         self._call(params, self.err_update)
 
     def branch(self):
-        params = ["git", "clone", "--depth", "1", self.origin, self.path]
-        self._call(params, self.err_branch, os.path.dirname(self.path))
+        params = ["git", "clone", "--depth", "1"]
+        # Deal with branches in the format
+        # <repository-url>;<branch>
+        components = self.origin.split(';')
+        if len(components) == 2:
+            params += ["--branch", components[1],
+                       components[0], self.path]
+        else:
+            params += [self.origin, self.path]
+        cwd = os.path.dirname(os.path.dirname(self.path))
+        if not cwd:
+            cwd = "."
+
+        self._call(params, self.err_branch, cwd)
+
+        change_ref = self.extended_options.get('changeref', None)
+        if change_ref:
+            change_ref = 'refs/changes/{}/{}'.format(
+                change_ref.split("/")[0][-2:], change_ref)
+
+            self._call(["git", "fetch", "--depth", "1",
+                        self.origin, change_ref],
+                       self.err_branch, self.path)
+            self._call(["git", "checkout", "FETCH_HEAD"], self.err_branch,
+                       self.path)
 
     def is_modified(self):
         params = ["git", "status", "-s"]
